/*                                                                      
    This file is part of KAddressBook.                                  
    Copyright (c) 2002 Mike Pilone <mpilone@slac.com>                   
                                                                        
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or   
    (at your option) any later version.                                 
                                                                        
    This program is distributed in the hope that it will be useful,     
    but WITHOUT ANY WARRANTY; without even the implied warranty of      
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the        
    GNU General Public License for more details.                        
                                                                        
    You should have received a copy of the GNU General Public License   
    along with this program; if not, write to the Free Software         
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           
                                                                        
    As a special exception, permission is given to link this program    
    with any edition of Qt, and distribute the resulting executable,    
    without including the source code for Qt in the source distribution.
*/                                                                      

#include <qcombobox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qwidgetstack.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qhbox.h>

#include <kapp.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>
#include <kabc/field.h>

#include "undo.h"
#include "undocmds.h"
#include "contactentrylist.h"
#include "viewmanager.h"
#include "configureviewdialog.h"
#include "viewwrapper.h"
#include "iconviewwrapper.h"
#include "tableviewwrapper.h"
#include "addviewdialog.h"
#include "jumpbuttonbar.h"

////////////////////////////////////////
// View Manager

ViewManager::ViewManager(ContactEntryList *doc, KConfig *config, 
                         QWidget *parent, const char *name)
  : QWidget(parent, name)
{
  mConfig = config;
  mDocument = doc;
  
  // Create the GUI
  initGUI();
  
  // Set the list to auto delete the views and the wrappers
  mViewDict.setAutoDelete(true);
  mViewWrapperDict.setAutoDelete(true);
  
  // Create the view wrappers
  createViewWrappers();
  
  mActiveView = 0;
}

ViewManager::~ViewManager()
{
  unloadViews();
  mViewWrapperDict.clear();
}

void ViewManager::readConfig()
{
  // No need to tell each view to read its config, since
  // this will happen as the views are created.
    
  // Read the view names
  mConfig->setGroup("Views");
  mViewNameList = mConfig->readListEntry("Names");
    
  if (mViewNameList.size() == 0)
  {
    // Add a default
    mViewNameList << "Default Table View";
  }
}

void ViewManager::writeConfig()
{
    // Iterator through all the views
    QDictIterator<KAddressBookView> iter(mViewDict);
    for (iter.toFirst(); iter.current(); ++iter)
    {
        mConfig->setGroup(iter.currentKey());
        (*iter)->writeConfig(mConfig);
    }
    
    // write the view name list
    mConfig->setGroup("Views");
    mConfig->writeEntry("Names", mViewNameList);
}

QStringList ViewManager::selectedUids()
{
  return mActiveView->selectedUids();
}

void ViewManager::sendMail()
{
  QString emailAddrs = mActiveView->selectedEmails();
  kapp->invokeMailer( emailAddrs, "" );
}

  
void ViewManager::deleteAddressee()
{
  kdDebug()<< "ViewManager::deleteAddressee\n";
  
  ContactEntry *ce;
  
  // Get the selected uids
  QStringList uidList = mActiveView->selectedUids();
  
  QStringList::Iterator iter;
  for (iter = uidList.begin(); iter != uidList.end(); ++iter)
  {
      // For each one, create a delete command and push it on the
      // stack
      ce = mDocument->find(*iter);
      
      PwDeleteCommand *command = new PwDeleteCommand(mDocument, *iter, 
                                                     ce );
      UndoStack::instance()->push( command );
      RedoStack::instance()->clear();
  }
  
  // now if we deleted anything, refresh
  if (uidList.size() > 0)
  {
      mActiveView->refresh();
      emit modified();
  }
}

void ViewManager::paste()
{
  QClipboard *cb = QApplication::clipboard();
  PwPasteCommand *command = new PwPasteCommand( mDocument, cb->text() );
  UndoStack::instance()->push( command );
  RedoStack::instance()->clear();
  
  modified();
  mActiveView->refresh();
}

void ViewManager::copy()
{
    QStringList uidList = mActiveView->selectedUids();
    ContactEntry *ce;
    QString clipText;
    QTextOStream clipStream(&clipText);
    
    QStringList::Iterator iter;
    for (iter = uidList.begin(); iter != uidList.end(); ++iter)
    {
        ce = mDocument->find(*iter);
        if (ce != 0L)
            ce->save( clipStream );
    }
    
  QClipboard *cb = QApplication::clipboard();
  cb->setText( clipText );
}

void ViewManager::cut()
{
  PwCutCommand *command = new PwCutCommand(mDocument, 
                                           mActiveView->selectedUids());
  UndoStack::instance()->push( command );
  RedoStack::instance()->clear();
  
  mActiveView->refresh();
  emit modified();
}

void ViewManager::setSelected(QString uid, bool selected)
{
  mActiveView->setSelected(uid, selected);
}

void ViewManager::unloadViews()
{
  mViewDict.clear();
  mActiveView = 0;
}

void ViewManager::setActiveView(const QString &name)
{
    // Find the view
    KAddressBookView *view = 0;
    
    // Check that this isn't the same as the current active view
    if (mActiveView && (mActiveView->name() == name))
        return;
        
    // At this point we know the view that should be active is not
    // currently active. We will try to find the new on in the list. If
    // we can't find it, it means it hasn't been instantiated, so we will
    // create it on demand.
    
    view = mViewDict.find(name);
    
    // Check if we found the view. If we didn't, then we need to create it
    if (view == 0)
    {
      KConfig *config = kapp->config();
      config->setGroup(name);
      QString type = config->readEntry("Type", "Table");
      
      kdDebug() << "ViewManager::setActiveView: creating view - "
                << name << endl;
      
      // Find the wrapper, ask it to create the view
      ViewWrapper *wrapper = mViewWrapperDict.find(type);
      if (wrapper)
          view = wrapper->createView(mDocument, mViewWidgetStack, 
                                     name.latin1());
                                     
      if (view)
      {
        mViewDict.insert(name, view);
        mViewWidgetStack->addWidget(view);
        view->readConfig(config);
        
        // The manager just relays the signals
        connect(view, SIGNAL(selected(const QString &)),
                this, SIGNAL(selected(const QString &)));
        connect(view, SIGNAL(executed(const QString &)),
                this, SIGNAL(executed(const QString &)));
        connect(view, SIGNAL(modified()),
                this, SIGNAL(modified()));
      }
    }
    
    // If we found or created the view, raise it and refresh it
    if ( view )
    {
      mActiveView = view;
      mViewWidgetStack->raiseWidget(view);
      view->refresh();
      
      // Update the inc search combo to show the fields in the new active
      // view.
      refreshIncrementalSearchCombo();
    }
    else
    {
      kdDebug() << "ViewManager::setActiveView: unable to find view\n";
    }
}

void ViewManager::refresh(QString uid)
{
    mActiveView->refresh(uid);
}

void ViewManager::modifyView()
{
  // Find the wrapper for the type they are modifying
  ViewWrapper *wrapper;
  ConfigureViewDialog *dialog = 0;
  
  // Find the wrapper
  wrapper = mViewWrapperDict.find(mActiveView->type());
  
  if (wrapper)
      dialog = wrapper->createConfigureViewDialog(mActiveView->name(),
                                                  this, "ConfigureViewDialog");
  
  // If we found the wrapper and it successfully created a dialog, disllay it
  if (dialog)
  {
    // Set the config group
    KConfig *config = kapp->config();
    config->setGroup(mActiveView->name());
    dialog->readConfig(config);
    
    // Let the dialog run (it is modal)
    if (dialog->exec())
    {
      // The user accepted
      dialog->writeConfig(config);
      mActiveView->readConfig(config);
      mActiveView->refresh();
      
      // cleanup
      delete dialog;
    }
  }
}
    
void ViewManager::deleteView()
{
  // Confirm with the user
  QString text = i18n("Are you sure that you want to delete the view \"%1\"?").arg( mActiveView->name() );
  
  QString caption = i18n("Confirm Delete");
  
  if (KMessageBox::questionYesNo(this, text, caption) == KMessageBox::Yes)
  {
    mViewNameList.remove(mActiveView->name());
    mViewDict.remove(mActiveView->name());
    mActiveView = 0;
    
    // we are in an invalid state now, but that should be fixed after
    // we emit the signal
    emit viewConfigChanged(QString::null);
  }
}

void ViewManager::addView()
{
  // Display the add view dialog
  AddViewDialog dialog(&mViewWrapperDict, this, "AddViewDialog");
  
  if (dialog.exec())
  {

    QString newName = dialog.viewName();
    QString type = dialog.viewType();
    
    // Check for name conflicts
    bool firstConflict = true;
    int numTries = 1;
    while (mViewNameList.contains(newName) > 0)
    {
      if (!firstConflict)
      {
        newName = newName.left(newName.length()-4);
        firstConflict = false;
      }
            
      newName.sprintf("%s <%d>", newName.latin1(), numTries);
      numTries++;
    }
    
    // Add the new one to the list
    mViewNameList << newName;
    
    // write the view to the config file,
    // launch the view config dialog
    KConfig *config = kapp->config();
    config->deleteGroup(newName);   // Incase they had this view before
    config->setGroup(newName);
    config->writeEntry("Type", type);
    
    // try to set the active view
    emit viewConfigChanged(newName);
    
    // Now let the user modify it
    modifyView();
  }
}

void ViewManager::createViewWrappers()
{
  ViewWrapper *wrapper;
  
  // View Developers: Add an entry here to create the wrapper for your view
  // type and add it to the list. Thats it :D
  
  wrapper = new IconViewWrapper();
  mViewWrapperDict.insert(wrapper->type(), wrapper);
  
  wrapper = new TableViewWrapper();
  mViewWrapperDict.insert(wrapper->type(), wrapper);
}

void ViewManager::initGUI()
{
  
    QBoxLayout *layout = new QVBoxLayout( this );
  
   // Add the incremental search fields
    mIncrementalSearchWidget = new QWidget(this, "mIncrementalSearchWidget");
    QHBoxLayout *incLayout = new QHBoxLayout(mIncrementalSearchWidget);
    incLayout->setMargin( 2 );
    incLayout->setSpacing( 3 );
    
    QLabel *label = new QLabel(i18n("&Incremental Search:"), 
                               mIncrementalSearchWidget);
    incLayout->addWidget(label);
    
    mIncrementalSearchEdit = new QLineEdit(mIncrementalSearchWidget, 
                                           "mIncrementalSearchEdit");
    label->setBuddy(mIncrementalSearchEdit);
    incLayout->addWidget(mIncrementalSearchEdit);
    
    mIncrementalSearchCombo = new QComboBox(mIncrementalSearchWidget, 
                                            "mIncrementalSearchCombo");
    incLayout->addWidget(mIncrementalSearchCombo);
    
    layout->addWidget(mIncrementalSearchWidget);
    layout->setStretchFactor(mIncrementalSearchWidget, 0);
    
    QHBox *hBox = new QHBox(this, "hBox");
    layout->addWidget(hBox);
    layout->setStretchFactor(hBox, 1);
    
    // The widget stack for the different views
    mViewWidgetStack = new QWidgetStack(hBox, "mViewWidgetStack");
    hBox->setStretchFactor(mViewWidgetStack, 1);
    
    // Add the jump bar
    mJumpButtonBar = new JumpButtonBar(hBox, "mJumpButtonBar");
    hBox->setStretchFactor(mJumpButtonBar, 0);
    
    // Connect the slots and signals
    connect(mIncrementalSearchEdit, SIGNAL(textChanged(const QString &)),
            this, SLOT(incrementalSearchTextChanged(const QString &)));
    connect(mIncrementalSearchEdit, SIGNAL(returnPressed()),
            this, SLOT(incrementalSearchReturnPressed()));
    connect(mIncrementalSearchCombo, SIGNAL(activated(const QString &)),
            this, SLOT(incrementalSearchComboActivated(const QString &)));
    connect(mJumpButtonBar, SIGNAL(jumpToLetter(const QChar &)),
            this, SLOT(jumpToLetter(const QChar &)));
}


void ViewManager::refreshIncrementalSearchCombo()
{
    // Clear the combo
    mIncrementalSearchCombo->clear();
    
    // Insert all the items
    // Note: There is currently a problem with i18n here. If we translate each
    // item, the user gets the right display, but then the incrementalSearch
    // function is called with the translated text, which is incorrect. 
    // Hmm.. how to fix? -mpilone
    KABC::Field::List fields = mActiveView->fields();

    mIncrementalSearchFields.clear();

    KABC::Field::List::ConstIterator it;
    int i = 0;
    for( it = fields.begin(); it != fields.end(); ++it ) {
      mIncrementalSearchCombo->insertItem( (*it)->label(), i );
      mIncrementalSearchFields.append( *it );
      ++i;
    }
}

void ViewManager::incrementalSearchTextChanged(const QString &)
{
    // run the incremental search
    incrementalSearchReturnPressed();
}

void ViewManager::incrementalSearchReturnPressed()
{
    // Get the values from the widgets and run the search
    mActiveView->incrementalSearch( mIncrementalSearchEdit->text(),
        mIncrementalSearchFields[ mIncrementalSearchCombo->currentItem() ] );
}

void ViewManager::incrementalSearchComboActivated(const QString &)
{
    // run the incremental search
    incrementalSearchReturnPressed();
}

void ViewManager::jumpToLetter(const QChar &ch)
{
  // Jumping always works based on the first field
  mActiveView->incrementalSearch(QString(ch),
        mIncrementalSearchFields[ mIncrementalSearchCombo->currentItem() ] );
}


void ViewManager::setIncrementalSearchVisible(bool visible)
{
    if (visible)
      mIncrementalSearchWidget->show();
    else
      mIncrementalSearchWidget->hide();
}


void ViewManager::setJumpButtonBarVisible(bool visible)
{
    if (visible)
      mJumpButtonBar->show();
    else
      mJumpButtonBar->hide();
}

#include "viewmanager.moc"
